import os
from .settings import PLUGIN_NAME

socket_dir = '/run/docker/plugins'
socket_file = PLUGIN_NAME + '.sock'
socket_path = os.path.join(socket_dir, socket_file)

def main():
    os.execvp(
        'gunicorn',
        [
            'gunicorn',
            '--bind=unix://{}'.format(socket_path),
            '--workers=1',
            'wolphin_driver.wsgi:app'
        ]
    )



import os, sys, re
from envparse import env

BASE_DIR = env.str('BASE_DIR')
SAFE_DELETION = env.bool('SAFE_DELETION', default=True)
PLUGIN_NAME = env.str('PLUGIN_NAME', default='wolphin-driver')

if not os.path.isdir(BASE_DIR):
    err = '{} directory is set as base directory, but it doesn\'t exist'.format(BASE_DIR)
    sys.exit(err)

if not os.access(BASE_DIR, os.W_OK):
    err = 'No write permission for base directory {}'.format(BASE_DIR)
    sys.exit(err)

plugin_name_regexp = re.compile('^[A-Za-z0-9][A-Za-z0-9\_\-]{0,30}$')

if not plugin_name_regexp.match(PLUGIN_NAME):
    err = 'Wrong plugin name: {}'.format(PLUGIN_NAME)
    sys.exit(err)




import os, shutil, re
from flask import Flask, request, jsonify
from .settings import (
    BASE_DIR,
    SAFE_DELETION,
)


app = Flask(__name__)
name_regexp = re.compile('^[a-zA-Z0-9][a-zA-Z0-9\_\.\-]{0,40}$')


def full_path(name):
    return os.path.join(BASE_DIR, name)


@app.route('/Plugin.Activate', methods=['POST'])
def plugin_activate():
    return jsonify(
        {
            'Implements': ['VolumeDriver']
        }
    )


@app.route('/VolumeDriver.Create', methods=['POST'])
def create():
    data = request.get_json(force=True)
    path = full_path(data['Name'])

    if os.path.isdir(path):
        err = 'Volume is already created'
    else:
        try:
            os.mkdir(path)
        except Exception as e:
            err = str(e)
        else:
            err = ''

    return jsonify(
        {
            'Err': err
        }
    )



@app.route('/VolumeDriver.Remove', methods=['POST'])
def remove():
    data = request.get_json(force=True)
    name = data['Name']
    path = full_path(name)

    try:
        if SAFE_DELETION:
            new_name = '_' + name
            new_path = full_path(new_name)
            shutil.move(path, new_path)
        else:
            shutil.rmtree(name)
    except Exception as e:
        err = str(e)
    else:
        err = ''

    return jsonify(
        {
            'Err': err
        }
    )


@app.route('/VolumeDriver.Path', methods=['POST'])
@app.route('/VolumeDriver.Mount', methods=['POST'])
def mount():
    data = request.get_json(force=True)
    path = full_path(data['Name'])

    if os.path.isdir(path):
        err = ''
    else:
        err = 'Volume doesn\'t exist'

    return jsonify(
        {
            'Mountpoint': path,
            'Err': err
        }
    )


@app.route('/VolumeDriver.Unmount', methods=['POST'])
def unmount():
    data = request.get_json(force=True)
    path = full_path(data['Name'])

    if os.path.isdir(path):
        err = ''
    else:
        err = 'Volume doesn\'t exist'

    return jsonify(
        {
            'Err': err
        }
    )


@app.route('/VolumeDriver.Get', methods=['POST'])
def get():
    data = request.get_json(force=True)
    name = data['Name']
    path = full_path(name)

    if os.path.isdir(path):
        err = ''
    else:
        err = 'Volume doesn\'t exist'

    return jsonify(
        {
            'Volume': {
                'Name': name,
                'MountPoint': path,
                'Status': {}
            },
            'Err': err
        }
    )


@app.route('/VolumeDriver.List', methods=['POST'])
def list():
    try:
        names = [
            name for name in os.listdir(BASE_DIR)
            if name_regexp.match(name)
        ]

        volumes = []
        for name in names:
            path = full_path(name)

            if os.path.isdir(path):
                volumes.append(
                    {
                        'Name': name,
                        'MountPoint': path
                    }
                )
    except Exception as e:
        return jsonify(
            {
                'Err': str(e)
            }
        )

    return jsonify(
        {
            'Volumes': volumes,
            'Err': ''
        }
    )


@app.route('/VolumeDriver.Capabilities', methods=['POST'])
def capabilities():
    return jsonify(
        {
            'Capabilities': {
                'Scope': 'global'
            }
        }
    )

if __name__ == "__main__":
    app.run(host='0.0.0.0')



#!/usr/bin/env python3

from setuptools import setup

readme = open('README.md').read()

setup(
    name = 'wolphin-driver',
    version = '0.0.2',
    description = "Docker custom base directory volume driver",
    long_description = readme,
    long_description_content_type='text/markdown',
    author = "Wolphin",
    author_email = "wolphin@wolph.in",
    url = "https://gitlab.com/q_wolphin/wolphin-driver",
    packages = ['wolphin_driver'],
    classifiers = [
        'Natural Language :: English',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7'
    ],
    install_requires=[
        'flask',
        'gunicorn',
        'envparse'
    ],
    entry_points={
        'console_scripts': [
            'wolphin-driver=wolphin_driver:main',
        ]
    },
)

